#!/usr/bin/env python3
#-*- coding: utf-8 -*-

#
# (c) Rene Hadler, iteas IT Services GmbH
# rene.hadler@iteas.at
# www.iteas.at
#

from subprocess import STDOUT, check_output, TimeoutExpired
import os
import signal
import sys
import time

TIMEOUT=600
FILE="/run/getmail.lock"
RCDIR="/etc/getmailrc"

def signal_handler(sig, frame):
    print("Got signal %s, exiting." % sig)
    remove_lock()
    sys.exit(0)

def remove_lock():
    if os.path.exists(FILE):
        os.remove(FILE)

signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)
signal.signal(signal.SIGHUP, signal_handler)

# Remove lock if lock file is older than 1 hour to keep system running
if os.path.exists(FILE):
    st = os.stat(FILE)
    mtime = int(st.st_mtime)
    if((time.time() - mtime) > 3600):
        remove_lock()

if not os.path.exists(FILE):
    open(FILE, 'a').close()

    rcfiles = [f for f in os.listdir(RCDIR) if os.path.isfile(os.path.join(RCDIR, f)) and f.startswith("rc-")]
    cconf = ""
    for rcfile in rcfiles:
        cconf += " --rcfile " + rcfile

    try:
        print(check_output("getmail -g %s %s" % (RCDIR, cconf), stderr=STDOUT, timeout=TIMEOUT, shell=True).decode("UTF-8"))
    except TimeoutExpired:
        print("Error: Getmail timed out")
    except Exception as e:
        print("Error: Unexpected exception raised", e)
    finally:
        remove_lock()