#!/usr/bin/env python3
#
# prints active kopano users
##############################

import argparse
import ldap
import sys
import configparser

config = configparser.ConfigParser()
config.read('/etc/ucs-iteas/main.conf')

BASE_DN = config.get("LDAP", "BASE_DN")
LDAP_SERVER = config.get("LDAP", "LDAP_SERVER")
LDAP_B_USER = config.get("LDAP", "LDAP_B_USER")
LDAP_B_PASS = config.get("LDAP", "LDAP_B_PASS")

parser = argparse.ArgumentParser(description='Outputs all UCS users with an active kopano mailbox')
parser.add_argument('-c', '--count', action='store_true', default=False, help="Only output user count")
args = parser.parse_args()

basedn = BASE_DN
searchFilter = "(&(univentionObjectType=users/user)(kopanoAccount=1)(!(kopanoSharedStoreOnly=1)))"
bind_u = LDAP_B_USER
bind_p = LDAP_B_PASS

l = ldap.initialize(LDAP_SERVER)

try:
	l.protocol_version = ldap.VERSION3
	l.simple_bind_s(bind_u,bind_p)
except:
	sys.exit("Error while binding to LDAP: ")

try:
	res = l.search_s(basedn, ldap.SCOPE_SUBTREE, searchFilter, ['uid'])
	if args.count == True:
		print(len(res))
		sys.exit()

	for user in res:
		print(user[1]['uid'][0].decode('utf-8'))
except:
	sys.exit("Error querying ldap: ")

