#!/usr/bin/env python
# Moves items from SRCFOLDER to DSTFOLDER

import kopano
from datetime import datetime
from MAPI.Util import *

folder_count = 0
item_count = 0
from_date = datetime.strptime('1970-01-01', '%Y-%m-%d')
until_date = datetime.strptime('2038-01-19', '%Y-%m-%d')

def main():
	global folder_count
	global item_count
	global from_date
	global until_date

	parser = kopano.parser('skpfucmUP')
	parser.add_option("-r", "--recursive", dest="recursive", action="store_true", help="Recursive; include subfolders.")
	parser.add_option("--src", "--source", dest="srcfolder", action="store", help="Source folder (e.g. 'Trash', 'Inbox', 'Inbox/subfolder')")
	parser.add_option("--dst", "--destination", dest="dstfolder", action="store", help="Destination folder (e.g. 'Trash', 'Inbox', 'Inbox/subfolder')")
	parser.add_option("--dst-in-root", dest="rootdst", action="store_true", help="DSTFOLDER is in the root-folder, not under Inbox")
	parser.add_option("--create", dest="create", action="store_true", help="Create path of SRCFOLDER in DSTFOLDER and move items there")
	parser.add_option("--from", dest="fromdate", action="store", help="(YYYY-MM-DD) Only move items no older than this. Implies --one-by-one when RECURSIVE is set as we cannot move whole folders with this option.")
	parser.add_option("--until", dest="untildate", action="store", help="(YYYY-MM-DD) Only move items older than this. Implies --one-by-one when RECURSIVE is set as we cannot move whole folders with this option.")
	parser.add_option("--one-by-one", dest="onebyone", action="store_true", help="Only affects RECURSIVE mode. Will move items in subfolders one-by-one instead of whole subfolders at once. Can be useful when unable to move subfolders, eg. if subfolders already exist in DSTFOLDER. Leaves empty subfolders in SRCFOLDER and significantly decreases performance.")
	options, args = parser.parse_args()

	recursive = False

	if not options.users or not options.srcfolder or not options.dstfolder:
		print("Usage: {} [-r] -u <username> --src <source folder> --dst <destination folder> [--from <YYYY-MM-DD>] [--until <YYYY-MM-DD>] [--create] [--one-by-one]".format(sys.argv[0]))
		sys.exit(1)
	
	if options.recursive:
		recursive = True
	
	server = kopano.Server(options)
	user = server.user(options.users[0])
	store = user.store
	srcbox = store.folder(options.srcfolder)

	if options.rootdst:
		dstbox = store.subtree
		if options.dstfolder:
			dstbox = dstbox.folder(options.dstfolder)
	else:
		dstbox = store.folder(options.dstfolder)

	if options.create:
		if not dstbox.get_folder(options.srcfolder):
			dstbox.create_folder(options.srcfolder)
		dstbox = dstbox.get_folder(options.srcfolder)

	folder_count = 0
	item_count = 0

	if options.fromdate:
                from_date = datetime.strptime(options.fromdate, '%Y-%m-%d')
	if options.untildate:
                until_date = datetime.strptime(options.untildate, '%Y-%m-%d')

	if recursive:
		if(options.fromdate or options.untildate or options.onebyone):
			move_folder_contents(options, srcbox, dstbox)
		else:
			for subfolder in srcbox.folders():
				path_parts = subfolder.path.split("/")
				path_parts.pop()
				sub_path = "/".join(path_parts)
				if sub_path != srcbox.path:
					continue

				if dstbox.get_folder(subfolder.name):
					print "Warning: Subfolder '{}' already exists in destination! Cannot move. To work around this limitation, delete the subfolder in the destination or use --one-by-one option".format(subfolder.name)
					continue
				srcbox.move(subfolder, dstbox)
				folder_count += 1
			
			for item in srcbox:
				item.move(dstbox)
				item_count += 1
	else:
		for item in srcbox:
			if item.received == None:
				received_date = item.created
			else:
				received_date = item.received
			if options.fromdate and received_date < from_date:
				continue
			if options.untildate and received_date > until_date:
				continue
			item.move(dstbox)
			item_count += 1

	if recursive:
		print "Moved {} subfolder(s) and {} item(s)".format(folder_count, item_count)
	else:
		print "Moved {} item(s)".format(item_count)


def move_folder_contents(options, topfolder, dstfolder):
	global folder_count
	global item_count
	global from_date
	global until_date
	
	for item in topfolder:
		if item.received == None:
			received_date = item.created
		else:
			received_date = item.received

		if received_date >= from_date and received_date <= until_date:
			item_count += 1
			item.move(dstfolder) ## make sure move to right subfolder

	subfolders = topfolder.folders()
	if subfolders:
		for subfolder in subfolders:
			# remove last part of subfolder.path and compare to topfolder.path
			path_parts = subfolder.path.split("/")
			path_parts.pop()
			sub_path = "/".join(path_parts)

			if sub_path != topfolder.path:
				continue
			if not dstfolder.get_folder(subfolder.name):
				dstfolder.create_folder(subfolder.name)
			folder_count += 1
			move_folder_contents(options, subfolder, dstfolder.get_folder(subfolder.name))
	
	return True


if __name__ == "__main__":
    main()

