#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# (c) Rene Hadler iteas IT Services GmbH
# support@iteas.at
# www.iteas.at
#

import kopano
import json
import base64
import ldap
import sys
import pprint
from MAPI.Util import *
import OpenSSL.crypto
import configparser
import datetime

config = configparser.ConfigParser()
config.read('/etc/ucs-iteas/main.conf')

BASE_DN = config.get("LDAP", "BASE_DN")
LDAP_SERVER = config.get("LDAP", "LDAP_SERVER")
LDAP_B_USER = config.get("LDAP", "LDAP_B_USER")
LDAP_B_PASS = config.get("LDAP", "LDAP_B_PASS")
LDAP_EXT_ATTR_NAME = config.get("LDAP_ATTR", "SHAREDSTORES_NAME")

print("Starting update_shared_stores at %s" % datetime.datetime.now())

pp = pprint.PrettyPrinter(indent=4)

kserver = kopano.Server()

l = ldap.initialize(LDAP_SERVER)
l.protocol_version=ldap.VERSION3
#l.set_option(ldap.OPT_X_TLS_REQUIRE_CERT,ldap.OPT_X_TLS_NEVER)
#l.set_option(ldap.OPT_X_TLS_NEWCTX,0)
l.simple_bind_s(LDAP_B_USER, LDAP_B_PASS)

# Get attr for custom shared stores
datakattr = l.search_s("cn=univention," + BASE_DN, ldap.SCOPE_SUBTREE, "(&(objectClass=univentionUDMProperty)(cn=%s))" % LDAP_EXT_ATTR_NAME, ["univentionUDMPropertyLdapMapping"])
if(len(datakattr) > 1 and len(datakattr) != 0):
    print("Mehr als ein Attribut gefunden, breche ab ")
    sys.exit(1)

ssattr = datakattr[0][1]["univentionUDMPropertyLdapMapping"][0]

"""
Read user settings

:param user: The user
:return: Settings
"""
def read_settings(user):
    try:
        mapisettings = user.store.prop(PR_EC_WEBACCESS_SETTINGS_JSON).value.decode('utf-8')
        settings = json.loads(mapisettings)
    except Exception as e:
        print('{}: Has no or no valid WebApp settings creating empty config tree'.format(user.name))
        settings = json.loads('{"settings": {"zarafa": {"v1": {"contexts": {"mail": {}}}}}}')
    return settings

"""
Write WebApp setting into the user store

:param user: The user
:param setting: The setting that should be written
"""
def write_settings(user, setting):
    try:
        user.store.create_prop(PR_EC_WEBACCESS_SETTINGS_JSON, setting.encode('utf-8'))
    except Exception as e:
        print('{}: Error Writing WebApp settings for user: {}'.format(e, user.name))

def main():
    data = l.search_s(BASE_DN, ldap.SCOPE_SUBTREE, "(&(kopanoAccount=1)(|(kopano4ucsRole=user)(kopano4ucsRole=admin)))", ["uid", ssattr])
    for user in data:
        uid = user[1]["uid"][0]
        if len(user[1]) > 1:
            sharedstores = user[1][ssattr]
            print("Geteilte Mailboxen gefunden fuer", uid, sharedstores)
            user = kserver.get_user(uid)
            if(user != None):
                settings = read_settings(user)
                if not "hierarchy" in settings["settings"]["zarafa"]["v1"]["contexts"]:
                    settings["settings"]["zarafa"]["v1"]["contexts"]["hierarchy"] = {}

                if not "shared_stores" in settings["settings"]["zarafa"]["v1"]["contexts"]["hierarchy"]:
                    settings["settings"]["zarafa"]["v1"]["contexts"]["hierarchy"]["shared_stores"] = {}

                sstores = settings["settings"]["zarafa"]["v1"]["contexts"]["hierarchy"]["shared_stores"]
                itersstores = dict(settings["settings"]["zarafa"]["v1"]["contexts"]["hierarchy"]["shared_stores"])

                # Remove shared mailboxes if managed by this script
                for exmbox, exmboxobj in itersstores.iteritems():
                    if not exmbox in sharedstores:
                        if "itauto" in exmboxobj and exmboxobj["itauto"] == True:
                            sstores.pop(exmbox)
                            print(" - Mailbox %s wurde fuer User %s entfernt" % (exmbox, uid))

                # Add shared mailboxes
                for stname in sharedstores:
                    if not stname in sstores:
                        shmbox = {}
                        shmbox["all"] = {}
                        shmbox["all"]["folder_type"] = "all"
                        shmbox["all"]["show_subfolders"] = False

                        sstores[stname] = shmbox
                        sstores[stname]["itauto"] = True

                        print(" + Mailbox %s wurde fuer User %s hinzugeguegt" % (stname, uid))
                    else:
                        print(" ~ Mailbox %s ist bereits User %s zugewiesen" % (stname, uid))

                write_settings(user, json.dumps(settings))

        else:
            print("Keine geteilten Mailboxen gefunden fuer", uid)

if __name__ == "__main__":
    main()